unit RegFileEd;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, RegExt, ComCtrls, ExtCtrls, ImgList, RegChangeIcon, Menus,
  uEditFlags, uCommand, RegFileId;

type
  TfrmFileType = class(TForm)
    edId: TEdit;
    Label1: TLabel;
    edDesc: TEdit;
    Label2: TLabel;
    edIcon: TEdit;
    Label3: TLabel;
    chbQView: TCheckBox;
    Image1: TImage;
    Bevel1: TBevel;
    Button1: TButton;
    Label4: TLabel;
    cbExt: TComboBox;
    lvCommands: TListView;
    Label5: TLabel;
    Button3: TButton;
    Label6: TLabel;
    edEditFlags: TEdit;
    Button4: TButton;
    Bevel2: TBevel;
    PopupMenu1: TPopupMenu;
    New1: TMenuItem;
    Edit1: TMenuItem;
    Delete1: TMenuItem;
    SetDefault1: TMenuItem;
    btnLoad: TButton;
    RegFileType1: TRegFileType;
    dlgIconRes: TIconResourceDialog;
    Label7: TLabel;
    procedure Button3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure edDescChange(Sender: TObject);
    procedure cbExtChange(Sender: TObject);
    procedure chbQViewClick(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Edit1Click(Sender: TObject);
    procedure Delete1Click(Sender: TObject);
    procedure lvCommandsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lvCommandsCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure SetDefault1Click(Sender: TObject);
    procedure edIdChange(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure RegFileType1CopyType(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FfrmTypeId   : TfrmTypeId;
    shell32      : String;
    procedure  LoadCommands;
    procedure  EnableItems;
  public
    { Public declarations }
  end;

var
  frmFileType: TfrmFileType;

implementation
uses ShellAPI;

{$R *.DFM}


procedure TfrmFileType.Button3Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmFileType.FormCreate(Sender: TObject);
var
  szSystemDir: Array[0..MAX_PATH] of Char;
  i : UINT;
begin
  FfrmTypeId := nil;
  if GetSystemDirectory(szSystemDir,SizeOf(szSystemDir)) <>0 then
    shell32 := StrPas(szSystemDir)+'\shell32.dll';
  i := 0;
  Image1.Picture.Icon.Handle := ExtractIcon(Handle,PChar(shell32),i);
  EnableItems;
end;

procedure TfrmFileType.Button1Click(Sender: TObject);
var
  i : UINT;
begin
  dlgIconRes.IconPath := edIcon.Text;
  if dlgIconRes.Execute then
  begin
    edIcon.Text := dlgIconRes.IconPath;
    i := UINT(dlgIconRes.IconIndex);
    Image1.Picture.Icon.Handle := ExtractIcon(Handle,PChar(dlgIconRes.FileName),i);
    if Image1.Picture.Icon.Handle=0 then
    begin
      i := 0;
      Image1.Picture.Icon.Handle := ExtractIcon(Handle,PChar(shell32),i);
    end;
    RegFileType1.DefaultIcon := dlgIconRes.IconPath;
  end;
end;

procedure TfrmFileType.FormDestroy(Sender: TObject);
begin
  if FfrmTypeId<>nil then FfrmTypeId.Free;
  FfrmTypeId := nil;
end;

procedure TfrmFileType.Button4Click(Sender: TObject);
begin
  frmEditFlags.EditFlags :=  RegFileType1.EditFlags;
  if frmEditFlags.ShowModal=mrOk then
  begin
    RegFileType1.EditFlags := frmEditFlags.EditFlags;
    edEditFlags.Text := '0x'+EditFlagsAsHex(RegFileType1.EditFlags);
  end;
end;

procedure TfrmFileType.edDescChange(Sender: TObject);
begin
  RegFileType1.Desc := edDesc.Text;
end;

procedure TfrmFileType.cbExtChange(Sender: TObject);
begin
  RegFileType1.ShowExt := TRegShowExt(cbExt.ItemIndex);
end;

procedure TfrmFileType.chbQViewClick(Sender: TObject);
begin
  RegFileType1.QuickView := chbQView.Checked;
end;

procedure  TfrmFileType.EnableItems;
var
  Flag : Boolean;
begin
  Flag := edId.Text<>'';
  edDesc.Enabled     := Flag;
  Button1.Enabled    := Flag;
  Button4.Enabled    := Flag;
  cbExt.Enabled      := Flag;
  chbQView.Enabled   := Flag;
  lvCommands.Enabled := Flag;
end;

procedure TfrmFileType.LoadCommands;
var
  i : Integer;
  LI : TListItem;
begin
  lvCommands.Items.Clear;
  for i:=0 to RegFileType1.ShellCommands.Count-1 do
  With RegFileType1.ShellCommands.Commands[i] do
  begin
    LI := lvCommands.Items.Add;
    LI.Data := RegFileType1.ShellCommands.Commands[i];
    LI.Caption := Verb;
    LI.SubItems.Add(CommandName);
    LI.SubItems.Add(CommandLine);
    if UseDDE then LI.SubItems.Add('+')
              else LI.SubItems.Add('');
  end;
end;

procedure TfrmFileType.PopupMenu1Popup(Sender: TObject);
var
  S : Boolean;
begin
  S := lvCommands.Selected<>nil;
  Edit1.Enabled := S;
  Delete1.Enabled := S;
  SetDefault1.Enabled := S;
end;

procedure TfrmFileType.New1Click(Sender: TObject);
var
  C : TRegShellCommand;
  LI : TListItem;
begin
  With frmAction do
  begin
    Verb := '';
    CommandName := '';
    CommandLine := '';
    UseDDE      := False;
    FillChar(DDE,SizeOf(DDE),0);
    if ShowModal=mrOK then
    begin
      C := TRegShellCommand.Create(Verb);
      C.CommandName := CommandName;
      C.CommandLine := CommandLine;
      C.UseDDE      := UseDDE;
      if UseDDE then C.DDEExec := DDE;
      RegFileType1.ShellCommands.Add(C);
      LI := lvCommands.Items.Add;
      LI.Data := C;
      LI.Caption := C.Verb;
      LI.SubItems.Add(CommandName);
      LI.SubItems.Add(CommandLine);
      if UseDDE then LI.SubItems.Add('+')
                else LI.SubItems.Add('');
    end;
  end
end;

procedure TfrmFileType.Edit1Click(Sender: TObject);
var
  C : TRegShellCommand;
  LI : TListItem;
begin
  C := nil;
  LI := lvCommands.Selected;
  if (LI<>nil) then C := lvCommands.Selected.Data;
  if C=nil then Exit;
  With frmAction do
  begin
    Verb := C.Verb;
    CommandName := C.CommandName;
    CommandLine := C.CommandLine;
    UseDDE      := C.UseDDE;
    DDE         := C.DDEExec;
    if ShowModal=mrOK then
    begin
      C.Verb        := Verb;
      C.CommandName := CommandName;
      C.CommandLine := CommandLine;
      C.UseDDE      := UseDDE;
      if UseDDE then
      begin
        C.DDEExec := DDE;
        LI.SubItems[2] := '+';
      end
      else LI.SubItems[2] := '';
      LI.Caption := C.Verb;
      LI.SubItems[0] := CommandName;
      LI.SubItems[1] := CommandLine;
    end;
  end
end;

procedure TfrmFileType.Delete1Click(Sender: TObject);
var
  C : TRegShellCommand;
  LI : TListItem;
begin
  LI := lvCommands.Selected;
  if (LI<>nil) then
  begin
    C := lvCommands.Selected.Data;
    if C<>nil then RegFileType1.ShellCommands.DeleteCommand(C);
    lvCommands.Items.Delete(LI.Index);
  end;
end;

procedure TfrmFileType.lvCommandsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_DELETE then Delete1Click(nil);
end;

procedure TfrmFileType.lvCommandsCustomDrawItem(Sender: TCustomListView;
  Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
begin
  if (Item<>nil) and (Item.Data<>nil) then
  begin
    if RegFileType1.ShellCommands.IsDefault[Item.Index] then
      lvCommands.Canvas.Font.Style := lvCommands.Canvas.Font.Style + [fsBold];
  end;
end;

procedure TfrmFileType.SetDefault1Click(Sender: TObject);
var
  LI : TListItem;
begin
  LI := lvCommands.Selected;
  if (LI<>nil) and (LI.Data<>nil) then
  begin
    RegFileType1.ShellCommands.DefaultCommand := LI.Data;
    lvCommands.Repaint;
  end;
end;

procedure TfrmFileType.edIdChange(Sender: TObject);
begin
  RegFileType1.Id := edId.Text;
end;

procedure TfrmFileType.btnLoadClick(Sender: TObject);
begin
  if FfrmTypeId=nil then
    FfrmTypeId := TfrmTypeId.Create(Self);
  FfrmTypeId.Id := RegFileType1.Id;
  if FfrmTypeId.ShowModal=mrOK then
  begin
    RegFileType1.Id := FfrmTypeId.Id;
    RegFileType1.Load;
    RegFileType1CopyType(Self);
  end;
  EnableItems;
end;

procedure TfrmFileType.RegFileType1CopyType(Sender: TObject);
var
  i   : UINT;
begin
  if RegFileType1=nil then Exit;
  dlgIconRes.IconPath := RegFileType1.DefaultIcon;
  edIcon.Text := RegFileType1.DefaultIcon;
  i := UINT(dlgIconRes.IconIndex);
  Image1.Picture.Icon.Handle := ExtractIcon(Handle,PChar(dlgIconRes.FileName),i);
  if Image1.Picture.Icon.Handle=0 then
  begin
    i := 0;
    Image1.Picture.Icon.Handle := ExtractIcon(Handle,PChar(shell32),i);
  end;
  edId.Text := RegFileType1.Id;
  edDesc.Text := RegFileType1.Desc;
  edEditFlags.Text :='0x'+EditFlagsAsHex(RegFileType1.EditFlags);
  cbExt.ItemIndex := Integer(RegFileType1.ShowExt);
  chbQView.Checked := RegFileType1.QuickView;
  LoadCommands;
end;

procedure TfrmFileType.FormShow(Sender: TObject);
begin
  FocusControl(btnLoad);
end;

end.
